/*Copyright (C) 2019 Centro de Computacao Cientifica e Software Livre
Departamento de Informatica - Universidade Federal do Parana

This file is part of Plataforma Integrada MEC.

Plataforma Integrada MEC is free software: you can redistribute it and/or modify
it under the terms of the GNU Affero General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

Plataforma Integrada MEC is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU Affero General Public License for more details.

You should have received a copy of the GNU Affero General Public License
along with Plataforma Integrada MEC.  If not, see <http://www.gnu.org/licenses/>.*/

import React, { useState, useContext } from 'react';
//imports material ui componets
import Card from "@material-ui/core/Card";
import CardContent from "@material-ui/core/CardContent";
import CardAction from '@material-ui/core/CardActions';
import { Typography, TextField, Button, Grid } from '@material-ui/core';
import CircularProgress from '@material-ui/core/CircularProgress';
import AddRoundedIcon from '@material-ui/icons/AddRounded';
import ListRoundedIcon from '@material-ui/icons/ListRounded';
//imports local files
import SnackBar from '../../../../Components/SnackbarComponent';
import { Store } from '../../../../Store';
import { postRequest } from '../../../../Components/HelperFunctions/getAxiosConfig'
import Unauthorized from '../Unauthorized';
//router
import { Link } from 'react-router-dom';

const CreateInstitution = (props) => {
    const { state } = useContext(Store);

    const [name, setName] = useState('Nova Instituição');
    const [description, setDescription] = useState('');
    const [adress, setAdress] = useState('');
    const [city, setCity] = useState('');
    const [country, setCountry] = useState('');

    const [isLoading, setIsLoading] = useState(false)

    // Handle error in name
    const [errorInName, setErrorInName] = useState({
        error: false,
        message: '',
    })

    // Handle error in Country 
    const [errorInCountry, setErrorInCountry] = useState({
        error: false,
        message: '',
    })

    const [snackInfo, setSnackInfo] = useState({
        message: '',
        icon: '',
        open: false,
        color: '',
    })

    const NameHandler = (e) => {
        setName(e.target.value)
        if (errorInName.error) {
            setErrorInName({
                error: false,
                message: ''
            })
        }
    }
    const DescriptionHandler = (e) => {
        setDescription(e.target.value)
    }
    const AdressHandler = (e) => {
        setAdress(e.target.value)
    }
    const CityHandler = (e) => {
        setCity(e.target.value)
    }
    const CountryHandler = (e) => {
        if (errorInCountry.error) {
            setErrorInCountry({
                error: false,
                message: ''
            })
        }
        setCountry(e.target.value)
    }

    // Handle snack infos
    const HandleSnack = (message, state, icon, color) => {
        setSnackInfo({
            message: message,
            icon: icon,
            open: state,
            color: color
        })
    }

    const CheckUserPermission = () => {
        let canUserEdit = false;

        if (state.userIsLoggedIn) {
            const roles = [...state.currentUser.roles];
            for (let i = 0; i < roles.length; i++)
                if (roles[i].name === 'admin' || roles[i].name === 'editor')
                    canUserEdit = true;
        }
        else {
            canUserEdit = false;
        }

        return canUserEdit;
    }


    //Handle submit 
    async function onSubmit() {
        setIsLoading(true)
        const api = '/institutions'
        const body = {
            "institution": {
                'name': name,
                'description': description,
                'address': adress,
                'city': city,
                'country': country,
            }
        }
        postRequest(
            api,
            body,
            (data) => {
                if (data.id)
                    HandleSnack('A instituição foi criada com sucesso', true, 'success', '#228B22')
                else {
                    if (data.errors) {
                        HandleSnack(`${data.errors[0]}`, true, 'warning', '#FA8072')
                    }
                    else {
                        if (data.name) {
                            let nameError = "";
                            data.name.map((msg) => (
                                nameError = nameError + msg + " e "
                            ))
                            setErrorInName({
                                error: true,
                                message: nameError
                            })
                        }
                        HandleSnack('Você precisa preencher algumas informações obrigatórias', true, 'warning', '#FFC125')
                    }
                }
                setIsLoading(false)
            },
            (error) => {
                HandleSnack('Ocorreu algum erro', true, 'warning', '#FA8072')
                setIsLoading(false)
            }
        )
    }

    // Fields
    const fields = [
        {
            label: 'Nome',
            value: name,
            required: true,
            error: errorInName.error,
            errorMessage: errorInName.message,
            onChange: (event) => NameHandler(event)
        },
        {
            label: 'Descrição',
            value: description,
            required: false,

            onChange: (event) => DescriptionHandler(event)
        },
        {
            label: 'Endereço',
            value: adress,
            required: false,

            onChange: (event) => AdressHandler(event)
        },
        {
            label: 'Cidade',
            value: city,
            required: false,
            onChange: (event) => CityHandler(event)
        },
        {
            label: 'País',
            value: country,
            required: false,
            error: errorInCountry.error,
            errorMessage: errorInCountry.message,
            onChange: (event) => CountryHandler(event)
        }
    ]

    if (CheckUserPermission()) {
        return (
            <Card>
                <SnackBar
                    severity={snackInfo.icon}
                    text={snackInfo.message}
                    snackbarOpen={snackInfo.open}
                    color={snackInfo.color}
                    handleClose={() => setSnackInfo({
                        message: '',
                        icon: '',
                        open: false,
                        color: ''
                    })}
                />
                <CardContent>
                    <Grid container direction='row' justify='space-between' alignContent="center" alignItems="center" xs={12}>
                        <Grid item>
                            <Typography variant='h4'>
                                {name}
                            </Typography>
                        </Grid>
                        <Grid item>
                            <Link style={{ textDecoration: 'none' }} to={'/admin/intitutions'}>
                                <Button
                                    onClick={props.BackToList}
                                    startIcon={<ListRoundedIcon />}
                                    variant='outlined'
                                    color='primary'
                                >
                                    Listar
                                </Button>
                            </Link>
                        </Grid>
                    </Grid>

                    <div style={{ height: '1em' }}></div>

                    <form style={{ display: 'flex', flexDirection: 'column' }}>
                        {fields.map((field, index) => (
                            <TextField
                                key={index}
                                required={field.required}
                                error={field.error}
                                helperText={field.error ? field.errorMessage : ''}
                                style={{ width: '250px', marginBottom: '1em' }}
                                label={field.label}
                                value={field.value}
                                onChange={field.onChange}
                                type="search"
                                multiline={true}
                            />
                        ))}
                    </form>
                </CardContent>
                <CardAction>
                    <Button
                        onClick={() => {
                            onSubmit();
                        }}
                        variant="contained"
                        color="primary"
                        disabled={isLoading}
                        startIcon={isLoading ? null : <AddRoundedIcon />}
                    >
                        {
                            isLoading ? <CircularProgress size={24} /> : 'Adicionar'
                        }
                    </Button>
                </CardAction>
            </Card>
        );
    } else return <Unauthorized />
}

export default CreateInstitution;